; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

(define (script-fu-dungeon-shadows image drawable height direction density shadow-opacity shadow-blur 
			shadow-blur-radiusx shadow-blur-radiusy shadow-blur-method color )
  
	(let* ((height (truncate height))
         (direction (truncate (modulo (- 180 direction) 360)))
		 (varLayerName "")
         (density (truncate density))
         (blur-layer 0)
         (dup-layer 0)
         (orig-sel 0)
	)
	(gimp-context-push)
	(gimp-image-undo-group-start image)
	
	(set! varLayerName (car (gimp-item-get-name drawable)))
	(set! orig-sel (car (gimp-selection-save image)))
	(set! blur-layer (car (gimp-layer-copy drawable TRUE)))
	
	;logging
    ;(gimp-message-set-handler ERROR-CONSOLE)
    ;(gimp-message-set-handler CONSOLE)
    ;(gimp-message-set-handler MESSAGE-BOX)
	
    (gimp-context-set-foreground color)
	(gimp-image-insert-layer image blur-layer 0 -1)
    (gimp-layer-resize-to-image-size blur-layer)
    (if (zero? (car (gimp-selection-is-empty image)))
      (begin
        (gimp-selection-invert image)
        (gimp-edit-clear blur-layer)
        (gimp-selection-invert image)
        (gimp-edit-fill blur-layer FOREGROUND-FILL)
        (gimp-selection-none image))
      (begin
        (gimp-layer-set-lock-alpha blur-layer TRUE)
        (gimp-edit-fill blur-layer FOREGROUND-FILL)
        (gimp-layer-set-lock-alpha blur-layer FALSE)))
	
    (plug-in-mblur RUN-NONINTERACTIVE image blur-layer 0 height direction 0 0)
    (while (> density 0)
      (set! dup-layer (car (gimp-layer-copy blur-layer TRUE)))
      (gimp-image-insert-layer image dup-layer 0 -1)
      (set! blur-layer (car (gimp-image-merge-down image dup-layer CLIP-TO-IMAGE)))
      (set! density (- density 1)))
    (gimp-item-set-name blur-layer (string-append varLayerName " Shadows"))	
	
	;check shadow-blur. apply blur if true
	(if (= shadow-blur TRUE)
          (begin
		  ; Determine Blur Method (Defaults are reversed)
			(cond ((= shadow-blur-method 0)
					 (set! shadow-blur-method 1)  ; Set blur method to RLE
				  )
				  (else
					 (set! shadow-blur-method 0)  ; Set blur method to IIR
				  )
			)

		  ;(gimp-message "blurring")
			 (plug-in-gauss                    ; Gaussian blur the shadow (radius blur)
						1                      ; Non-interactive 
						image                  ; Image to apply blur 
						blur-layer	           ; Layer to apply blur
						shadow-blur-radiusx    ; Blur Radius x  
						shadow-blur-radiusy    ; Blue Radius y 
						shadow-blur-method)    ; Method (IIR=0 RLE=1)
	))
	(gimp-image-select-item image 2 orig-sel)			; Select original selection from saved channel
	(gimp-edit-clear blur-layer)						; Knockout selection
    (gimp-image-remove-channel image orig-sel)			; Remove saved Selection
    (gimp-layer-set-opacity blur-layer shadow-opacity)  ; Set Opacity
	 
	;done
	(gimp-image-undo-group-end image)
	(gimp-displays-flush)
    (gimp-progress-end)
    (gimp-context-pop)
    ) 
  )

(script-fu-register "script-fu-dungeon-shadows"
  "Dungeon Shadows"
  "More accurate shadows for dungeon walls, original by saulgoode"
  "Minimalistix"
  "Minimalistix, original by saulgoode"
  "April 2014"
  "RGB*, GRAY*"
  SF-IMAGE    "Image"    			0
  SF-DRAWABLE "Drawable" 			0
  SF-ADJUSTMENT "Height"			'( 15 1 1000 1 10 0 1)
  SF-ADJUSTMENT "Direction"			'( -75 -360 360 1 10 0 1)
  SF-ADJUSTMENT "Density (1-100)"	'( 5 1 100 1 10 0 1 )
  SF-ADJUSTMENT "Shadow Opacity"	'(100 0 100 1 1 0 0)
  SF-TOGGLE     "Blur Shadow?" 		TRUE                 
  SF-ADJUSTMENT "Blur Radius X"		'(5 1 120 1 1 0 0)
  SF-ADJUSTMENT "Blur Radius Y"		'(5 1 120 1 1 0 0)
  SF-OPTION     "Blur Method"		'("RLE Method" "IIR Method")
  SF-COLOR 		"Color" 			'(0 0 0)
  )
(script-fu-menu-register "script-fu-dungeon-shadows"
  "<Image>/Filters"
  )
